;
;   Save Ramdisk to HD
;
;   Compile with FASM for Menuet
;

include 'macros.inc'

  use32
  org    0x0

  db     'MENUET01'             ; 8 byte id
  dd     0x01                   ; header version
  dd     START                  ; start of code
  dd     I_END                  ; size of image
  dd     0x1000                 ; memory for app
  dd     0x1000                 ; esp
  dd     0x0 ; , 0x0            ; I_Param , I_Icon


;******************************************************************************

  ID_YES = 0x10
  ID_NO  = 0x11
  ID_OK  = 0x12

  ST_IDLE   = 0
  ST_SAVING = 1
  ST_DONE   = 2

;******************************************************************************


START:                          ; start of execution

    mov  eax,40
    mov  ebx,101b
    int  0x40

red:
    call draw_window

still:
    mov  eax, 10                 ; wait here for event
    int  0x40

    dec  eax                  ; redraw request ?
    je   red
;    dec  eax
;    dec  eax                  ; button in buffer ?
;    je   button


  button:                      ; button
    mov  eax,17                ; get id
    int  0x40

    cmp  ah, ID_YES
    je   .yes
    cmp  ah, ID_NO
    je   .exit
    cmp  ah, ID_OK
    je   .exit

    dec  ah                    ; button id=1 ?
    jne  still

  .exit:
    or   eax,-1                    ; close this program
    int  0x40

  .yes:
    ; delete yes and no buttons
    mov   edx, ID_YES or 0x80000000
    mcall 8
    mov   edx, ID_NO  or 0x80000000
    mcall

    mov   [status], ST_SAVING
    call  draw_controls
    mcall 18, 6
;    mcall 5, 100

    mov   [status], ST_DONE
    call  draw_controls
    jmp   still



;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************

draw_window:

    mov  eax, 12                ; function 12:tell os about windowdraw
    mov  ebx, 1                 ; 1, start of draw
    int  0x40
                                ; DRAW WINDOW
    mov  eax, 0                 ; function 0 : define and draw window
    mov  ebx, 200*65536+246     ; [x start] *65536 + [x size]
    mov  ecx, 200*65536+72      ; [y start] *65536 + [y size]
    mov  edx, 0x03dddddd        ; color of work area RRGGBB,8->color gl
    int  0x40
                                ; WINDOW LABEL
    mov  eax, 4                 ; function 4 : write text to window
    mov  ebx, 8*65536+8         ; [x start] *65536 + [y start]
    mov  ecx, 0x10ffffff        ; font 1 & color ( 0xF0RRGGBB )
    mov  edx, header            ; pointer to text beginning
    mov  esi, header.size       ; text length
    int  0x40

    call draw_controls

    mov  eax,12                 ; function 12:tell os about windowdraw
    mov  ebx,2                  ; 2, end of draw
    int  0x40
ret



draw_controls:
    mcall 13, 10*65536+230, 24*65536+42, 0xDDDDDD
    mov   eax, [status]
    cmp   eax, ST_IDLE
    je    .idle
    cmp   eax, ST_SAVING
    je    .saving
    cmp   eax, ST_DONE
    je    .done
    jmp   .finish

  .idle:
    mcall 4, 12*65536+26, 0, question,  question.size
    mcall  , 12*65536+36,  , question2, question2.size
    mcall 8, 90*65536+30, 48*65536+14, ID_YES, 0xa0a0a0
    mcall  ,126*65536+30,            , ID_NO
if lang eq ru
    mcall 4, 100*65536+52, 0, yesbtn, yesbtn.size
    mcall  , 133*65536+52,  , nobtn,  nobtn.size
else
    mcall 4, 97*65536+52, 0, yesbtn, yesbtn.size
    mcall  , 136*65536+52,  , nobtn,  nobtn.size
end if
    jmp   .finish

  .saving:
    mcall 4, 12*65536+26, 0, saving, saving.size
    jmp   .finish

  .done:
    mcall 4, 12*65536+26, 0, written, written.size
    mcall 8, 108*65536+30, 43*65536+14, ID_OK, 0xa0a0a0
    mcall 4, 118*65536+47, 0, okbtn, okbtn.size

  .finish:
ret


; DATA AREA

sz  header, "RD2HD"

lsz question,\
    ru, " , ⮡ ࠭ ࠬ ",\
    en, "Click YES to save ramdisk to"

sz  question2, "/hd/1/menuet.img"

lsz yesbtn,\
    ru, "",\
    en, "Yes"

lsz nobtn,\
    ru, "",\
    en, "No"

lsz saving,\
    ru, " . , .",\
    en, "Please wait while saving your ramdisk."

lsz written,\
    ru, " ᯥ譮 ࠭.",\
    en, "Fisihed."

sz  okbtn, "OK"

status dd ST_IDLE

I_END:
