;---------------------------------------------------------------------
;
;   MenuetOS AC97 WAV Player
;
;   ஢ FASM'
;
;---------------------------------------------------------------------

  use32              ;  32- ० ᥬ
  org    0x0         ;   

  db     'MENUET01'  ; 8- 䨪 MenuetOS
  dd     0x01        ;   (ᥣ 1)
  dd     START       ;  ࢮ 
  dd     IMAGE_END   ; ࠧ ணࠬ
  dd     MEMORY_END  ; ⢮ 
  dd     STACK_P     ;  設 
  dd     0x0         ;    ࠬ஢ ( ᯮ)
  dd     0x0         ; १ࢨ஢

;---------------------------------------------------------------------

include "INCLUDES.INC"

ENDOFFILE = 0x1

;  Uncomment these strings if you don't want to receive debug messages:

;  macro dps str   {}
;  macro dpd num   {}
;  macro pregs     {}
;  macro newline   {}
;  macro print str {}
;  macro dph arg   {}

;---------------------------------------------------------------------
;---  MAIN PROGRAM  --------------------------------------------------
;---------------------------------------------------------------------

START:
        mcall   MF_PCI, 1
        test    al, al
        jnz     @f
        print   "Error: cannot access PCI bus. :-("
        jmp     exit
     @@:
        mov     [lastbus], al

        print   "I'm going now to find PCI device"

        mov     [bus], -1
  .next_bus:
        inc     [bus]
        mov     [devfn], 0
  .next_devfn:

        mov     cl, 0
        call    pciRegRead32

        cmp     eax, (ICH_DID shl 16) + INTEL_VID
        je      found_ich

        cmp     eax, (ICH0_DID shl 16) + INTEL_VID
        je      found_ich0

        cmp     eax, (ICH2_DID shl 16) + INTEL_VID
        je      found_ich2

        cmp     eax, (ICH3_DID shl 16) + INTEL_VID
        je      found_ich3

        cmp     eax, (ICH4_DID shl 16) + INTEL_VID
        je      found_ich4

        cmp     eax, (MX440_DID shl 16) + INTEL_VID
        je      found_440mx

        cmp     eax, (SI7012_DID shl 16) + SIS_VID
        je      found_sis7012

        cmp     eax, (NFORCE_DID shl 16) + NVIDIA_VID
        je      found_nforce

        cmp     eax, (NFORCE2_DID shl 16) + NVIDIA_VID
        je      found_nforce2

        cmp     eax, (AMD8111_DID shl 16) + AMD_VID
        je      found_amd8111

        cmp     eax, (AMD768_DID shl 16) + AMD_VID
        je      found_amd768

        inc     [devfn]
        cmp     [devfn], 255
        jb      .next_devfn

        mov     al, [lastbus]
        cmp     [bus], al
        jb      .next_bus

        print   "Error: Could not find Intel ICH based AC97 audio device!"
        print   "Maybe your codec is disabled in BIOS?"
        print   "Exiting NOW!"
        jmp     exit


found_ich:
        dps    "Intel 82801AA (ICH)"
        jmp     proceed

found_ich0:
        dps    "Intel 82801AB (ICH0)"
        jmp     proceed

found_ich2:
        dps    "Intel 82801BA (ICH2)"
        jmp     proceed

found_ich3:
        dps    "Intel 82801CA (ICH3)"
        jmp     proceed

found_ich4:
        dps    "Intel 82801DB (ICH4)"
        mov     [AC97ICH4], 1
        jmp     proceed

found_440mx:
        dps    "Intel 82443MX (440MX)"
        jmp     proceed

found_sis7012:
        dps    "SiS SI7012"
        jmp     proceed

found_nforce:
        dps    "NVIDIA NFORCE"
        jmp     proceed

found_nforce2:
        dps     "NVIDIA NFORCE2"
        jmp     proceed

found_amd8111:
        dps    "AMD AMD8111"
        jmp     proceed

found_amd768:
        dps    "AMD AMD768"
        jmp     proceed

 proceed:
        print   " integrated AC97 audio codec detected."

        mov     cl, NAMBAR_REG
        call    pciRegRead16

        and     ax, 0xfffe
        mov     [NAMBAR], ax
        test    ax, ax
        jnz     mixer_base_ok

        print   "Error: Intel ICH based AC97 audio codec disabled in BIOS!"
        jmp     exit

 mixer_base_ok:
        dps     "NAMBAR: "
        dph     eax
        newline

        mov     cl, NABMBAR_REG
        call    pciRegRead16
        and     ax, 0xFFC0
        mov     [NABMBAR], ax
        test    ax, ax
        jnz     bm_base_ok

        print   "Error: Intel ICH based AC97 audio codec disabled in BIOS!"
        jmp     exit

 bm_base_ok:
        dps     "NABMBAR: "
        dph     eax
        newline

        mov     cl, IRQ_REG
        call    pciRegRead8
        mov     [AC97IRQ], al

        mov     cl, INT_REG
        call    pciRegRead8
        mov     [AC97INT], al

        cmp     [AC97ICH4], 1
        jne     skip_ich4_init
        mov     cl, ICH4_CFG_REG
        call    pciRegRead8
        or      al, 0x1
        mov     dl, al
        call    pciRegWrite8

 skip_ich4_init:
;---------------------------------------------------------------------
        mov     cl, PCI_CMD_REG
        call    pciRegRead8               ; read PCI command register
        mov     dl, al
        or      dl, IO_ENA+BM_ENA         ; enable IO and bus master
        call    pciRegWrite8

;---------------------------------------------------------------------

        print "Enabling access to ports..."

; enable access to ports
        movzx   ecx, [NAMBAR]
        mov     edx, ecx
        add     edx, NAM_SIZE
        mcall   46, 0
        test    eax, eax
        jz      @f
        print   "Error: couldn't enable access to ports"
        jmp     exit
     @@:

        movzx   ecx, [NABMBAR]
        mov     edx, ecx
        add     edx, NABM_SIZE
        mcall   46, 0
        test    eax, eax
        jz      @f
        print   "Error: couldn't enable access to ports"
        jmp     exit
     @@:

; setup the Codec (actually mixer registers)
        call    print_info

        mov     eax, 48000
        call    codecConfig               ; unmute codec, set rates.
        test    eax, eax
        jnz     @f
        print   "Error: cannot initialize AC97 device."
        jmp     fpexit
       @@:

        call    print_info

        print   "Congrutalations! Your device has been initialized properly!"


; register reset the DMA engine.
        mov     edx, PO_CR_REG  ; PCM out control register
        mov     al, RR          ; reset
        call    NABMBAR_write_byte


; write last valid index to 31 to start with.
; The Last Valid Index register tells the DMA engine where to stop playing.
        mov     al, 1
        call    setLastValidIndex


; create Buffer Descriptors List
        call    prepare_BDL


; open player window
        mcall   51, 1, thread, thread_stack


; wait for command
  .new_check:
        cmp     [status], ST_PLAY
        jne     @f
        call    play
      @@:
        cmp     [status], ST_STOP
        jne     @f
        call    stop
      @@:
        cmp     [status], ST_EXIT
        je      fpexit
        mcall   5, 7
        jmp     .new_check

; All set. Let's play some music.
;        call    play
fpexit:

; free ports
        movzx   ecx, [NAMBAR]
        mov     edx, ecx
        add     edx, NAM_SIZE
        mcall   46, 1

        movzx   ecx, [NABMBAR]
        mov     edx, ecx
        add     edx, NABM_SIZE
        mcall   46, 1

;---------------------------------------------------------------------

exit:

        mcall   -1


;---------------------------------------------------------------------
;---  FUNCTIONS  -----------------------------------------------------
;---------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; prepare_BDL - initializes BUFFER DESCRIPTORS LIST
prepare_BDL:
        mov     ecx, 32 / 2     ; make 32 entries in BDL
        mov     edi, BDL_BUFFER
        call    get_my_address
@@:
        ; set buf. desc. 0 to start of data file in memory
        push    eax
        add     eax, WAV_BUFFER1
        stosd
        ; set length to 60k samples. 1 sample is 16 bit or 2 bytes.
        xor     eax, eax
        add     eax, 60*1024   ; number of samples
        or      eax, BUP
        stosd

        mov     eax, [esp]
        add     eax, WAV_BUFFER2
        stosd
        xor     eax, eax
        add     eax, 60*1024
        or      eax, BUP
        stosd

        pop     eax
        loop    @b

        ; tell the DMA engine where to find our list of Buffer Descriptors.
        ; eax = base addr!
        mov     edx, PO_BDBAR_REG
        add     eax, BDL_BUFFER
        call    NABMBAR_write_dword
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; stop - stops current music
;; in:  nothing
;; out: nothing
stop:
        print   "STOP!"
        push    eax edx
        mcall   5, 10
        mov     edx, PO_CR_REG
        mov     al, 0
        call    NABMBAR_write_byte
        cmp     [status], ST_STOP
        jne     .exit
        mov     [status], ST_DONE
      .exit:
        pop     edx eax
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; play - plays wav file!
;; in:  nothing
;; out: nothing (but sound :)
play:
        print   "START!"
        push    eax edx

        mov     [fileinfo.first_block], 0 ; reset!
        call    update_first_buffer
        call    update_second_buffer

        mcall   58, fileinfo
        shr     ebx, 9
        mov     [file_size], ebx

        mov     edx, PO_CIV_REG
        mov     al, 1
        call    NABMBAR_write_byte

        mcall   5, 5

        call    updateLVI

        mov     edx, PO_CR_REG
        mov     al, RPBM
        call    NABMBAR_write_byte

   .tuneLoop:
     @@:
        mcall   5, 2
        call    getCurrentIndex
        test    al, BIT0
        jz      @b         ; loop if not ready yet

        call    updateLVI

        call    update_first_buffer
        test    al, al
        jnz     .exit

        cmp     [status], ST_PLAY
        jne     .exit

        test    [volume], 0x10000000
        je      @f
        mov     al, byte [volume]
        call    setVolume
        and     [volume], 0x7FFFFFFF
     @@:

     @@:
        mcall   5, 2
        call    getCurrentIndex
        test    al, BIT0
        jnz     @b         ; loop if not ready yet

        call    updateLVI

        call    update_second_buffer
        test    al, al
        jnz     .exit

        jmp     .tuneLoop
   .exit:
        call    stop
        pop     edx eax
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update_first_buffer - load a chunk into the first buffer, increments offset
;; in:  nothing
;; out: nothing
update_first_buffer:
        add     [fileinfo.first_block], 120
        mov     [fileinfo.dest], WAV_BUFFER1
        mcall   58, fileinfo
ret

update_second_buffer:
        add     [fileinfo.first_block], 120
        mov     [fileinfo.dest], WAV_BUFFER2
        mcall   58, fileinfo
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; get_my_address - get base address of the program in physical memory
;;   in:  nothing
;;   out: eax = address
get_my_address:
        pushad
        mcall   9, procinfo, -1
        popad
        mov     eax, [procinfo.memory_start]
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; set the last valid index to something other than we're currently playing
;; so that we never end
;;
;; this routine just sets the last valid index to 1 less than the index
;; that we're currently playing, thus keeping it in and endless loop
;; input:  none
;; output: none
updateLVI:
        push    eax
        call    getCurrentIndex
        dec     al
        and     al, INDEX_MASK
        call    setLastValidIndex
        pop     eax
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; returns AL = current index value
getCurrentIndex:
        push    edx
        mov     edx, PO_CIV_REG
        call    NABMBAR_read_byte
        pop     edx
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; input AL = index # to stop on
setLastValidIndex:
        push    edx
        mov     edx, PO_LVI_REG
        call    NABMBAR_write_byte
        pop     edx
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; print_info - outputs debug information
;;  in:  nothing
;;  out: nothing
print_info:
        newline

        dps     "BUS:              "
        movzx   eax, [bus]
        dph     eax
        newline

        dps     "DEVFN:            "
        movzx   eax, [devfn]
        dph     eax
        newline

        dps     "IRQ:              "
        movzx   eax, [AC97IRQ]
        dph     eax
        newline

        dps     "NAMBAR:           "
        movzx   eax, [NAMBAR]
        dph     eax
        newline

        dps     "NABMBAR:          "
        movzx   eax, [NABMBAR]
        dph     eax
        newline

        dps     "CODEC_POWER_CTRL: "
        mov     edx, CODEC_POWER_CTRL_REG
        call    NAMBAR_read_word
        dph     eax
        dps     " (bits 0-3 should be set)"
        newline

        mov     edx, 0x28
        call    NAMBAR_read_word
        dph     eax
        dps     " - supported features"
        newline
        add     edx, 0x2
        call    NAMBAR_read_word
        dph     eax
        dps     " - config"
        newline
        add     edx, 0x2
        call    NAMBAR_read_word
        dph     eax
        dps     " - PCM rate"

        newline
ret


;---------------------------------------------------------------------
;---     ----------------------------------------------
;---------------------------------------------------------------------
volume         dd 15

fileinfo:
 .mode         dd  0           ; READ
 .first_block  dd  0
 .blocks       dd  240         ; 120 Kb
 .dest         dd  file_data
 .work         dd  work_area
             ;  db  "/HD/1/WINDOWS/MEDIA/WICEB7~1.WAV",0
sz textbox_string, "/hd/1/music/file.wav",0

;---------------------------------------------------------------------

IMAGE_END:                     ; ⪠  ணࠬ
   rb 46-textbox_string.size
;---------------------------------------------------------------------

lastbus   db ?  ; pci coordinates
bus       db ?
devfn     db ?

AC97ICH4  db ?  ; Intel ICH4 codec flag
NAMBAR    dw ?  ; Audio Mixers Registers (base)
NABMBAR   dw ?  ; Bus Master Registers   (base)

AC97IRQ   db ?  ; Interrupt request
AC97INT   db ?  ; Interrupt pin

;---------------------------------------------------------------------

ST_DONE = 0x0
ST_PLAY = 0x1
ST_EXIT = 0x2
ST_STOP = 0x4

status    db ?

;---------------------------------------------------------------------

align 32
BDL_BUFFER:
 rb 256         ; 32 descriptors, 8 bytes each

;---------------------------------------------------------------------

file_data:

WAV_BUFFER1:
 rb 120 * 1024  ; 120 Kb

WAV_BUFFER2:
 rb 120 * 1024

;---------------------------------------------------------------------

procinfo process_information

work_area:
 rb 0x10000

;---------------------------------------------------------------------

rb 0x800
thread_stack:

rb 0x1000 ; for stack
STACK_P:
MEMORY_END: